import os

MY_DIR = os.path.dirname(__file__)
KOUGO_PATH = os.path.join(MY_DIR, 'kougo', 'kougo.txt.gz')
WEB_VERSES = os.path.join(MY_DIR, 'web', 't_web.csv.gz')
WEB_BOOKS = os.path.join(MY_DIR, 'web', 'key_english.csv.gz')
WEB_ABBRS = os.path.join(MY_DIR, 'web', 'key_abbreviations_english.csv.gz')
WEB_GENRES = os.path.join(MY_DIR, 'web', 'key_genre_english.csv.gz')
