__author__ = "Le Tuan Anh"
__email__ = "tuananh.ke@gmail.com"
__copyright__ = "Copyright 2018, freebible"
__credits__ = []
__license__ = "MIT License"
__description__ = "A bible processing toolkit with free bible data for Python"
__url__ = "https://github.com/neocl/freebible"
__maintainer__ = "Le Tuan Anh"
__version_major__ = "0.1"
__version__ = "{}a7".format(__version_major__)
__version_long__ = "{} - Alpha".format(__version_major__)
__status__ = "Prototype"
